use axum::{http::StatusCode, response::IntoResponse, Json};

use crate::types::error_code::ErrorCode;

use super::ErrorResponse;

#[derive(Debug, thiserror::Error)]
pub enum AuthenticationError {
    #[error("UserId is missing")]
    MissingUserId,
    #[error("The user ID is not a valid user name")]
    InvalidUserId,
    #[error("The provided authentication data was incorrect")]
    Forbidden,
    #[error("The user has been deactivated")]
    UserDeactivated,
}

impl IntoResponse for AuthenticationError {
    fn into_response(self) -> axum::response::Response {
        match self {
            Self::InvalidUserId | Self::MissingUserId => (
                StatusCode::BAD_REQUEST,
                Json(ErrorResponse::new(
                    ErrorCode::InvalidUsername,
                    &self.to_string(),
                    None,
                )),
            )
                .into_response(),
            Self::Forbidden => (
                StatusCode::FORBIDDEN,
                Json(ErrorResponse::new(
                    ErrorCode::Forbidden,
                    &self.to_string(),
                    None,
                )),
            )
                .into_response(),
            Self::UserDeactivated => (
                StatusCode::FORBIDDEN,
                Json(ErrorResponse::new(
                    ErrorCode::UserDeactivated,
                    &self.to_string(),
                    None,
                )),
            )
                .into_response(),
        }
    }
}
