use crate::types::server_name::ServerName;

pub struct Config {
    db_path: String,
    server_name: ServerName,
    enable_registration: bool
}

impl Config {
    /// Get a reference to the config's db path.
    #[must_use]
    pub fn db_path(&self) -> &str {
        self.db_path.as_ref()
    }

    /// Get a reference to the config's homeserver name.
    #[must_use]
    pub fn server_name(&self) -> &ServerName {
        &self.server_name
    }

    /// Get the config's enable registration.
    #[must_use]
    pub fn enable_registration(&self) -> bool {
        self.enable_registration
    }
}

impl Default for Config {
    fn default() -> Self {
        Self {
            db_path: "sqlite://db.sqlite3".into(),
            server_name: ServerName::new("fuckwit.dev").unwrap(),
            enable_registration: true
        }
    }
}