use super::{flow::Flow, identifier::Identifier};

#[derive(Debug, Clone, serde::Deserialize)]
#[serde(untagged)]
pub enum AuthenticationData {
    Password(AuthenticationPassword),
}

#[derive(Debug, Clone, serde::Deserialize)]
pub struct AuthenticationPassword {
    #[serde(rename = "type")]
    _type: Flow,
    identifier: Identifier,
    password: String,
    user: Option<String>,
    device_id: Option<String>,
    initial_device_display_name: Option<String>,
}

impl AuthenticationPassword {
    /// Get a reference to the authentication password's identifier.
    #[must_use]
    pub fn identifier(&self) -> &Identifier {
        &self.identifier
    }

    /// Get a reference to the authentication password's password.
    #[must_use]
    pub fn password(&self) -> &str {
        self.password.as_ref()
    }

    /// Get a reference to the authentication password's user.
    #[must_use]
    pub fn user(&self) -> Option<&String> {
        self.user.as_ref()
    }

    /// Get a reference to the authentication password's device id.
    #[must_use]
    pub fn device_id(&self) -> Option<&String> {
        self.device_id.as_ref()
    }

    /// Get a mutable reference to the authentication password's initial device display name.
    #[must_use]
    pub fn initial_device_display_name(&self) -> Option<&String> {
        self.initial_device_display_name.as_ref()
    }
}
