use sqlx::{encode::IsNull, Sqlite, Type};

#[derive(Debug, Clone)]
pub struct Uuid(pub uuid::Uuid);

impl Uuid {
    pub fn new_v4() -> Self {
        Uuid(uuid::Uuid::new_v4())
    }
}

impl Type<Sqlite> for Uuid {
    fn type_info() -> <Sqlite as sqlx::Database>::TypeInfo {
        <&str as Type<Sqlite>>::type_info()
    }
}

impl<'e> sqlx::Encode<'e, Sqlite> for Uuid {
    fn encode_by_ref(
        &self,
        buf: &mut <Sqlite as sqlx::database::HasArguments<'e>>::ArgumentBuffer,
    ) -> sqlx::encode::IsNull {
        buf.push(sqlx::sqlite::SqliteArgumentValue::Text(
            self.0.to_string().into(),
        ));

        IsNull::No
    }
}

impl<'d> sqlx::Decode<'d, Sqlite> for Uuid {
    fn decode(
        value: <Sqlite as sqlx::database::HasValueRef<'d>>::ValueRef,
    ) -> Result<Self, sqlx::error::BoxDynError> {
        let value = <&str as sqlx::Decode<Sqlite>>::decode(value)?;

        Ok(Uuid(uuid::Uuid::parse_str(value)?))
    }
}

impl From<uuid::Uuid> for Uuid {
    fn from(uuid: uuid::Uuid) -> Self {
        Uuid(uuid)
    }
}
